#if !defined(_CPLUSBASE_H)
#define _CPLUSBASE_H

#include "PubHeader.h"
#include "Base.h"
#include "Callback.h"
#include "Varient.h"
#include "Utility.h"
#include "Lex.h"
#include "ObGrid.h"

////////////////////////////////////////////////////////
#define SHOW_WHEN_COMMENT_NOT_FOUND  "unknow"

#define NW_TRAN_MODE_L_TO_H    1
#define NW_TRAN_MODE_H_TO_L    0
#define DEFAULT_NW_TRAN_MODE   NW_TRAN_MODE_L_TO_H


////////////////////////////////////////////////////////
////////////////////////////////////////////////////////

// GetTokenEx(nExStyle)
enum 
{ 
    EN_PROCIDENT    = 1 ,  // ֵ,  Func(...)
    EN_LINEREMARKRET= 2 ,
    EN_VALUESET     = 4 ,  //  { ... }
    EN_DOLLARITEM   = 8 ,  //  $Num
};

#ifdef _DLL_PROJECT
class CLASS_EXPORT CPlusLex: public CLex
#else
class CPlusLex: public CLex
#endif
{
public:
    enum register_type 
    { 
        REG_KEYWORD =0,    // KEYWORDʵĸϴ
        REG_OPERATOR=1,    // OperatorRelateOperator֧˫ַ
    };

public:
    CPlusLex();	
   ~CPlusLex();	

    bool  OpenFile(char* szfilename,bool bAsTextFile=true);
    char* GetName() { return m_BufFile.getFilename(); }
    char* GetPath() { return m_BufFile.getFilepath(); }
    char* GetAbsoluteFilename() { return m_BufFile.getAbsoluteFilename(); }
    
    void  Reset(bool bResetBuffer);
    
    /////////////////////////////////////////////////////////////////////
    // Extend Function, implement through GetToken()
    int   GetToken(LexToken& Token, bool bJustPreview=false);     // KeyWord
    int   GetToken(LexToken& Token, WORD nExStyle, bool bJustPreview);
    int   GetTokenEx(LexToken& Token, WORD nExStyle=0);             // չ: ֧Ident  . () [] :: 
                                                                    // REMARKS
    bool  GetSpecificToken(LexToken& Token,long  TokenId);          // ֻ֧TokenIdKeyWord, TokenId!=Remark, Remark
                                                                    // ɹ,λָͣԭ
    bool  GetSpecificToken(LexToken& Token,char* szTokenName,bool bcase=false);
     
    bool  Fetch_string(char* szLeftOperator, char* szRightOperator, // Ե() []
                       bool bReturnBetween, OTSTR& strReturn);      // شLeft,RightOperator

    bool  ForwardLocatTo(long nLocat);

    Location& getLocation();
    bool  setLocation(Location& locat);
    
private:
    BufferFile  m_BufFile;

    bool        m_bTokenValid;
    LexToken    m_Token;    // ɱһԪ, Preview
    
    ///////////////////////////////
    static LexRules     m_Rules;
    static StringArray  m_KeyWord_List;
    
    static bool Register(register_type type, char* lpstring);
    
public:
    //////////////////////////////////////////////////
    static void Initial();          // ʼʱһ
    static void Destroy();          // ʱһ

    static bool isValue(LexToken& Token);
    static bool isValueSet(LexToken& Token);
    static bool isKeyWord(LexToken& Token);
    static bool isIdent(LexToken& Token);   // չIdent
    static bool isOperator(LexToken& Token);
    static bool isProcIdent(LexToken& Token);
        
    static bool VarstringParser(char* varString,
                                OTSTR& strVar,OTSTR& strRestrict,OTSTR& strNext,
                                long* pRestrictLocat=NULL,long* pNextLocat=NULL);
};

//////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////
#ifdef _DLL_PROJECT
class CLASS_EXPORT ExprNode: public LexToken
#else
class ExprNode: public LexToken
#endif
{
public:
    enum { MAX_CHILD_NUM=3 };
    
public:
    //////////////////////////////////
    ExprNode* m_pChildList[MAX_CHILD_NUM];  // Ŀ 0, ˫Ŀ 0,1, Ŀ 0,1,2
    
public:
    ExprNode();
   ~ExprNode();
    
    bool isLeaf();
    int  GetHeight();
    
    void operator=(const ExprNode& right);
    
private:
    friend class ExprAnalyser;

    int  Height(ExprNode* pNode);
};

////////////////////
enum {
    EN_VALUSET_ASSIGN      = 1,
    EN_FIRST_VALUESET_ITEM = 2,
};

#ifdef _DLL_PROJECT
class CLASS_EXPORT ExprAnalyser  
#else
class ExprAnalyser  
#endif
{
public:
    class TResult: public CVarient
    {
    public:
        bool bisIdent;  // true   CVarientǱ(szIdent)
                        // false  CVarientǾֵ
    public:
        TResult() { bisIdent=false; }
       ~TResult() {}
        
        bool clone(TResult& ResultObject)
        {
            bisIdent=ResultObject.bisIdent;
            return CVarient::clone(ResultObject);
        }
        bool isIdent() 
        {
            return bisIdent;
        }
        void setIdent(char* szIdent)
        {
            bisIdent=true;
            set(szIdent);
        }
        char* getIdent()
        {
            char* szIdent;
            if (get(&szIdent)) return szIdent;
            return NULL;
        }
    };
    
public:
    ExprAnalyser();
   ~ExprAnalyser();
    
    ExprNode* GetExpression(char* lpstr_expression,WORD nFlag=0);
    ExprNode* GetExpression(CPlusLex& Parser, WORD nFlag=0); // ȡıʽ, Ϊձʽ
                                                                                // bEnableValueSetAssignָ鸳ֵ {...}
    ///////////////////////////////////////////////////////////////////
    void setCallback(void* handler, cCallback::tFunction pFunctionPointer);
    void setCallback(cCallback* pCallback);
    cCallback* getCallback();
    
    ///////////////////////////////////////////////////////////////////
    bool calculate(char* lpstr_expression,long&     result); // ʽֵ(ṹ͵ı,޷)
    bool calculate(char* lpstr_expression,OTSTR&    result);
    bool calculate(char* lpstr_expression,CVarient& result,bool bNullResultAllowed=false);  // bNullResultAllowedvoid
    bool calculate(ExprNode* pNode,long&     result);
    bool calculate(ExprNode* pNode,CVarient& result);
    
    bool execute(char* lpstr_expression);
    bool execute(ExprNode* pNode);
    bool execute(ExprNode* pNode,TResult& result); // ֵ
    
    ///////////////////////////////////////////////////////////////////
    static OTSTR GetExprstring(ExprNode* pNode,char* szParentOperator="#");
    static Location& GetExprLocat(ExprNode* pNode);
    static void OffsetExprLocat(ExprNode* pNode,Location& offset);
    static void GetExprNodes(ExprNode* pNode,TObArray<ExprNode>& List, bool bPrefixMode=true);  // ȡùpNodeExprNode
    static void GetExprSuffixstring(ExprNode* pNode,OTSTR& str);
        
    static void DelExprNode(ExprNode* pNode);

    char*  GetLastError() { return m_strLastError.GetBuffer(); }
    
private:
    OTSTR     m_strLastError;
    cCallback m_Callback;
    bool      m_bEnableValueSetAssign;
    bool      m_bEnableFirstValueSetItem;
    
    //////////////////////////////////////
    ExprNode* NewExprNode();
    
    ExprNode* BuildExprTree(CPlusLex& Parser,char* szPrecedeOperator,bool bFirstChildNode=true);
    ExprNode* ExtendNode   (CPlusLex& Parser,char* szPrecedeOperator, ExprNode* pNode); // if Found error, return NULL
    
    static int  GetOperatorType(char* pOperator,bool bAfterIdent=true);
    static int  GetOperatorPri (char* pOperator,bool bAfterIdent=true);
    
    bool GetValue   (LexToken* pToken,TResult& result);
    bool GetValue   (char* szIdent,TResult& result);
    bool AssignValue(LexToken* pToken,TResult& result);
    bool AssignValue(char* szIdent,TResult& result);
    bool AssignValueData(char* szIdent,char* szData);   // ڽṹֵ

    bool calculate(char* lpstr_expression,TResult& result,bool bNullResultAllowed=false);  // bNullResultAllowedvoid
    bool calculate(ExprNode* pNode,TResult& result);
    
    bool Check(ExprNode* pNode,bool bValueSetAllowed,Location* plocationOffset=NULL);  //### also should check ident validate
};

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
// Var Definition Object
#define CHOICE_TYPE   "Choice"
#define ANY_TYPE      "dynamic"

#ifdef _DLL_PROJECT
class CLASS_EXPORT VariableItemAttr: public TName
#else
class VariableItemAttr: public TName
#endif
{
public:
    enum type_  
    { 
        // basic type
        T_INT1   = 0,   T_INT2    = 1,   T_INT4  = 2, 
        T_UINT1  = 3,   T_UINT2   = 4,   T_UINT4 = 5,
        T_OCTET  = 7,   T_DOUBLE  = 8,
        T_PVOID  = 10, 
        T_PSTRUCT= 128, T_PCHOICE = 129,  // T_PVOID,  PCHOICEǺж̬Ľṹ
        T_NULL   = -1,  

        T_ANYTYPE= -2,  // ʾT_NULL(ֻprocedureͷñͷֵ֧)
        
        // ex type
        T_BYTE   = 20,  T_WORD    = 21,  T_DWORD = 22,
        T_STRING = 23,  // VisiableString
        T_SCHAR  = 24,  // char[n] used as VisiableString
        T_MEMO   = 25   // Large octet
    };

protected:
    type_ m_type;
    long  m_size;

    type_ m_Extype;
    OTSTR m_strTypeName;
    
    int   m_BitLen;     // Ըݸֵ, (type)Data, ֮ȡӦBits 
    int   m_BeginBit;
    bool  m_bisBitTail;
    
public:
    VariableItemAttr();
    virtual ~VariableItemAttr();
    
    bool  setAttr(char* varname,VariableItemAttr::type_ type,long size);
    
    VariableItemAttr::type_ getType();
    int   getSize();
    
    void  clone(VariableItemAttr& Object);
    void  clear();
    
    //////////////////////////////////////////////////////
    // Used for struct_typename and user defined typename
    void  setTypeName(char* typeName,VariableItemAttr::type_ extype);
    char* getTypeName();

    VariableItemAttr::type_ getExType();
    
    ////////////////////////////////////////////////////////////
    bool  setBitLen(int BitLen);
    int   getBitLen();
    
    bool  setBeginBit(int BeginBit);
    int   getBeginBit();
    
    void  setBitTailFlag(bool bTail);
    bool  getBitTailFlag();
    
    ////////////////////////////////////////////////////////////
    static bool isValidType(int type, bool bIncludeAnyType=true);
    static bool isBasicType(int type, bool bIncludeNull, bool bIncludePVOID=true);
    static bool isBasicType(char* szType, bool bIncludeNull, type_* returnType=NULL, type_* returnExType=NULL);

    static VariableItemAttr::type_ getBasicType(VariableItemAttr::type_ ExType);

    static char* getTypeDefaultName(int type); 
};

//////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////
class StructDefinition;

#ifdef _DLL_PROJECT
class CLASS_EXPORT VariableDefinition: public VariableItemAttr, public TObject
#else
class VariableDefinition: public VariableItemAttr, public TObject
#endif
{
public:
    WORD  m_nIdentifierId;
    bool  m_isNetWorkTranMode_L_TO_H;
    
protected:
    StructDefinition*  m_pParent;
    DWORD              m_wParam;                    // ڱ(ָ)
    
    StructDefinition*  m_pStructDefinitionObject;   // ṹ͵ı,ֵЧ
    bool               m_bPassAddress;              // Used for procedure params define
    OTSTR              m_strInitialValueData;       // ֵ

    OTSTR              m_strDescriptionEnd;

    long               m_Id;

public:
    VariableDefinition();
    VariableDefinition(StructDefinition* pParent,WORD nIdentifierId);
   ~VariableDefinition();

    int    ObjectTypeId() { return 4; }
   
    OTSTR  GetVariableName(bool bShowBeginBitLocat=true);
    StructDefinition* GetParent() { return m_pParent; }
    
    void   setParam(DWORD param) { m_wParam=param; }
    DWORD  getParam()  { return m_wParam; }

    ///////////////////////////////////////////////////
    void   setPassAddressFlag(bool bPassAddress) { m_bPassAddress=bPassAddress; }
    bool   getPassAddressFlag() { return m_bPassAddress; }

    void   setInitialValueData(char* szInitialValueData) { m_strInitialValueData=szInitialValueData; }
    OTSTR& getInitialValueData() { return m_strInitialValueData; }
    
    void   setDescriptionEnd(char* szDescription) { m_strDescriptionEnd=szDescription; }
    OTSTR& getDescriptionEnd() { return m_strDescriptionEnd; }
    
    void   setStructDefinitionObject(StructDefinition* pStructDefinition);  // ñǽṹ, Ч 
    StructDefinition* getStructDefinitionObject();      

    virtual StructDefinition* GetTypeOf() { return NULL; }  // ֻڼ򵥱
    
protected:
    friend class CVariableItem;
};


#ifdef _DLL_PROJECT
template class CLASS_EXPORT TObArray<VariableDefinition>;
#endif

#ifdef _DLL_PROJECT
class CLASS_EXPORT StructDefinition: public TName, public TObject
#else
class StructDefinition: public TName, public TObject
#endif
{
public:
    WORD  m_nIdentifierId;
    bool  m_isNetWorkTranMode_L_TO_H;

protected:
    DWORD m_wParam;
    
    TObArray<VariableDefinition>  m_VariableDefinitionList;
    
public:
    StructDefinition();
    StructDefinition(WORD nIdentifierId);
   ~StructDefinition();

    int   ObjectTypeId() { return 3; }
   
    void  setParam(DWORD param) { m_wParam=param; }
    DWORD getParam()  { return m_wParam; }
    
    int   GetVariableCount();
    
    void  Add(VariableDefinition* pObject,DWORD nParam=0);
    
    VariableDefinition* GetAt(int nIndex,DWORD* lpParam=NULL);
    VariableDefinition* GetVariableDefinition(char* name); 
    TObject*            Get_DefinitionObject(WORD nIdentifierId); 
    
    void  RemoveAll(bool bDeleteObject=true);
    
    bool  m_bLoopMark;

    virtual bool CompleteIt(OTSTR& strError);   // Check BitLen, and set BeginBit
    
    virtual bool GetIntegratedType(int* type=NULL) { return false; }
};

////////////////////////////////////////////////////
class CEnumItem: public CVarient, public TObject
{
public:
    CEnumItem() {}
   ~CEnumItem() {}

    int ObjectTypeId() { return 1; }
};

class EnumItemList: protected TObArray<CEnumItem>, public TObject
{
public:
    enum { COMMENT_OR=0, COMMENT_EQUAL=1 };

public:
    EnumItemList();
   ~EnumItemList();

    int    ObjectTypeId() { return 2; }
   
    void   setName(char* szName);
    OTSTR& getName();
    
    void   setDescription(char* szDescription);
    OTSTR& getDescription();
    
    /////////////////////////////////////////////////////
    int    GetSize();
    
    CEnumItem* AddItem(CEnumItem& EnumItem,bool bRepetitionCheck=true);
    int        FindItem(char* name);
    
    CEnumItem* GetAt(int nIndex);
    
    bool  Lookup_byName (char* name, long&  valReturn, char** szDescriptionReturn=NULL); 
    bool  Lookup_byValue(long value, OTSTR& strRetrun, char** szDescriptionReturn=NULL);
    
    void  RemoveAll();

    ////////////////////////////////////////////////////////////////
    bool  GetComments(long value,int nMode, OTSTR& strReturn,char* szDefault=NULL);
};

//////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////
// RunTime Object
//

// DumpToList(nDisplayAttr)
enum 
{ 
    SH_INTEGRATE_STRUC=1, SH_SHOW_VALUE_ANYWAY=2,  SH_SHOW_COMMENT=4, 
    SH_SHOW_ELEMID    =8, SH_SHOW_LEADERMARK  =16, 
    SH_INDENT_ON      =32, // 
};

#define COL_LOCATION     0
#define COL_BITMASK      1
#define COL_NAME         2
#define COL_DESCRIPTION  3
#define COL_COMMENT      4


class CStruct;
class RunEnv;
class NullSyntaxObject;

#ifdef _DLL_PROJECT
class CLASS_EXPORT CVariableItem: public TVarient
#else
class CVariableItem: public TVarient
#endif
{
    /////////////////////////////////////////
    // ע: TVarientдname,type,sizeЧ, m_Attr
    
public:
    CVariableItem();     
    virtual ~CVariableItem();

    bool   Create(CStruct* pParentStruct,VariableDefinition* pVariableDefinition);  // Ѷı
    bool   Create(StructDefinition* pStructDefinition, char* varname, long size);   // ѶĽṹ
    bool   Create(VariableItemAttr::type_ extype, char* varname, long size);        // ͱ
    
    bool   init()    { return false; }  // this function not allowed

    //////////////////////////////////////////////////
    VariableItemAttr&   GetAttr() { return m_Attr; }
    VariableDefinition* GetDefinition() { return m_pVariableDefinition; }
    
    int    getType() { return m_Attr.getType(); }
    int    getSize() { return m_Attr.getSize(); }
    OTSTR& getName() { return m_Attr.getName(); }
    char*  getTypeName()    { return m_Attr.getTypeName(); }
    int    getBitLen()      { return m_Attr.getBitLen();   }
        
    void   setDescription(char* szDescription) { m_Attr.setDescription(szDescription); }
    OTSTR& getDescription() { return m_Attr.getDescription(); }

    OTSTR  GetVariableName();  // name[..]
    OTSTR  GetNameEx();        // structname.itemname
    
    ////////////////////////////////////////////////
//  void   setParam(DWORD param) { m_wParam=param; }
//  DWORD  getParam()  { return m_wParam; }

    long   GetId() { return m_Id; }
    
    ////////////////////////////////////////////////
    // T_PCHOICET_PSTRUCT

    bool   isStruct(bool bIncludeChoice=true);

    long   GetSubElemCount(int nStructIdx,bool bConsiderImplicitChoiceElem);
    
    CVariableItem* GetSubElemAt(int nStructIdx, int nSubItemIdx, bool bConsiderImplicitChoiceElem);   // ȡnStructIdxṹԪصĵnSubItemIdx
    CVariableItem* FindFirstSubElembyId(int nStructIdx, long ElemId, bool bConsiderImplicitChoiceElem);   // : Structname DataA; ȡṹDataAµһ
    CVariableItem* FindFirstSubElem    (int nStructIdx, char* name,  bool bCaseSensitive,bool bConsiderImplicitChoiceElem); 
    CVariableItem* RemoveSubElemAt(int nStructIdx, int nSubItemIdx);

    ////////////////////////////////////////////////
    // T_PCHOICEǺж̬ṹ
    
    bool   isChoice();
    bool   isImplicitChoice();

    bool   AddChoiceElem(CVariableItem* pVariableItem, int ElemId, char* lpLeadChForDump=NULL); // : Choice DataA; ڿսṹDataA¼һ(ͿΪṹ)
    long   GetChoiceElemCount(bool bConsiderImplicitChoiceElem);

    CVariableItem* GetChoiceElemAt(int nSubItemIdx,     bool bConsiderImplicitChoiceElem);       // Choice IndexԶ=0, size=1
    CVariableItem* FindFirstChoiceElembyId(long ElemId, bool bConsiderImplicitChoiceElem);       // : Choice DataA; ȡṹDataAµһ
    CVariableItem* FindFirstChoiceElem    (char* name,  bool bCaseSensitive, bool bConsiderImplicitChoiceElem); 
    
    ////////////////////////////////////////////////
    virtual bool Clone(CVariableItem& VariableItem, bool bCloneNameAlso);   // bCloneNameAlso=false, ʾԭ
    virtual bool PassAddressClone(CVariableItem& SrcVariableItem,long nIndex,VariableDefinition* pVariableDefinition);  // صDefinition,Ҫƥ
    
    ////////////////////////////////////////////////
    // " (nIndex>= 0)||(size==1) szdataΪһԪ= ..  
    //   (nIndex==-1)&&(size> 1)       Ϊṹ    ={ .. , .. [,] }"
    bool   sets(long nIndex,  char* szdata, RunEnv* pCurRunEnv=NULL);   // pCurRunEnv ΪʽĻ
    bool   sets(char* szdata, RunEnv* pCurRunEnv=NULL);                 // nIndex=0, size=1Ԫصķ
    
    bool   setAt_WithBitsData(long nIndex, CVarient& VarientVal);
    bool   setAt_WithBitsData(long nIndex, unsigned  data);

    ////////////////////////////////////////////////
    // lastIdx=-1 mean noIdx or All array[], when lastIdx==NULL, һ[]

    // FOR CHOICE WITH NAME=NULL
    CVariableItem* GetItem(long nIdx,char* NextString, long* retlastIdx, RunEnv* pCurRunEnv=NULL);  
    CVariableItem* GetItem(char* NextString, long* retlastIdx, RunEnv* pCurRunEnv=NULL); 
    
    ////////////////////////////////////////////////
    bool   Dump(char* filename,bool bOverWrite);

    bool   DumpToList(VirtualList& List,TObject* pTranslater,WORD nDisplayAttr); // pTranslater used by GetValueComment()
    
protected:
    CStruct*            m_pParentStruct;
    VariableDefinition* m_pVariableDefinition;      // , Defined VariableDefinition 
    StructDefinition*   m_pStructDefinition;        // ṹ, ǽṹЧ
    VariableItemAttr    m_Attr;
    long                m_Id;                       // ʶ, Choice³ԱElemId
    DWORD               m_wParam;
    
    virtual CStruct* NewStruct();

    bool     SetStructItem(int nIndex,CStruct* pStruct);
    CStruct* GetStructItem(int nIndex);             // ע: VariableItemΪT_PSTRUCT,T_PCHOICE, ָCStruct
    
    bool Clone(CVariableItem& VariableItem,CStruct* pParentStruct, bool bCloneNameAlso);
    void Destroy();

    ///////////////////////////////////////////////////
    void Dump(FILE* fp, OTSTR strHeaderFormat);     // "%?ds  %?ds  %s"  (type name = data)

    bool DumpToList(CVariableItem* pParentObject,long MaxElemIdLen,VirtualList& List,long ParentItemId,TObject* pTranslater,WORD nDisplayAttr,long nSpaceLen, bool bOneOfMultiStructMember, bool bisFirstItemInStruct); 
    bool DumpStructToList(CVariableItem& VariableItem, int nIndex, VirtualList& List,long ParentItemId,TObject* pTranslater,WORD nDisplayAttr,long nSpaceLen,int nStructIndex,bool bShowChoiceItemMark); 
    
    bool SetListItemData(VirtualList& List,bool& bFirstRow, long& nItem, long nCol, DWORD Itemparam, char* szText);
    long GetMaxSubElemIdLen(int nStructIdx,bool bConsiderImplicitChoiceElem);
    
    virtual void DumpLocationMask(OTSTR& str) { str=""; }
    virtual bool DumpErrorFlag() { return false; }
    virtual bool ValueValid() { return true; }
    virtual bool DumpBitsMask(OTSTR& str) { str=""; return true; }

    ////////////////////////////////////////////////
    class InterData
    {
    public:
        char m_NameLeadCh;
        bool m_bisImplicitChoice;

    public:
        InterData()  { Clear(); }
       ~InterData()  {}
       
        void Clear()
        {
            m_NameLeadCh=' ';
            m_bisImplicitChoice=false;
        }
        void operator=(const InterData& right)
        {
            m_NameLeadCh=right.m_NameLeadCh;
            m_bisImplicitChoice=right.m_bisImplicitChoice;
        }
    };
    InterData  m_InterData;
    InterData* m_pPassAddressInterData;

public:
    void SetNameLeadCh(char Ch);
    char GetNameLeadCh() { return m_InterData.m_NameLeadCh; }

    ////////////////////////////////////////////////////////////
    // ֵComment: VarDefinition_Dump > TypeOF's struct's Dump
    virtual bool isValueCommentValid() { return false; }  
    virtual int  GetValueComment(TObject* pTranslater,CVariableItem* pParentObject,int nArrIndex,OTSTR& strComment,char* szDefault=SHOW_WHEN_COMMENT_NOT_FOUND) { strComment=""; return 0; } // falseʾδȡƥComment ( orequalƥ , strCommentɷزƥ)

    ////////////////////////////////////////////////////////////
    // TypeOfData: ȡۺֵĽṹ
    virtual bool isTypeOfDataValid(WORD nDisplayAttr) { return false; }
    virtual CVariableItem* GetTypeOfData(int nIndex,WORD nDisplayAttr) { return NULL; } 
    
private:
    friend class RunTimeStk;
    friend class CStruct;
};


///////////////////////////////////////////
class CStruct: public TName
{
public:
    CStruct();
    virtual ~CStruct();

    int  GetItemCount(bool bConsiderImplicitChoiceElem);
   
    CVariableItem* GetVariable  (char* name, bool bCaseSensitive, bool bConsiderImplicitChoiceElem); 
    CVariableItem* GetVariableAt(int nIndex, bool bConsiderImplicitChoiceElem); 
    CVariableItem* RemoveVariableAt(int nIndex);
    
    bool sets(char* szdata,RunEnv* pCurRunEnv=NULL);    // " { .. } "  pCurRunEnv ΪʽĻ, ڴӻȡӦֵ

    ///////////////////////////////////////////////////
    // lastIdx=-1 mean noIdx or All array[], when lastIdx==NULL, һ[]

    // FOR CHOICE WITH NAME=NULL
    CVariableItem* GetItem(char* varString, long* retlastIdx,RunEnv* pCurRunEnv=NULL);  
    
    ////////////////////////////////////////////////
//  void   setParam(DWORD param) { m_wParam=param; }
//  DWORD  getParam()  { return m_wParam; }
    
protected:
    StructDefinition*       m_pStructDefinition;        // Defined StructDefinition
    TObArray<CVariableItem> m_VariableItemList;
    bool                    m_bHasImplicitChoiceElem;
    DWORD                   m_wParam;
    
    bool Create(StructDefinition* pStructDefinition,char* szName=NULL);
    bool Clone (CStruct& StructItem);

    void Destroy();
     
    bool AddVariable(CVariableItem* pVariableItem);     // can be used For Choice

    CVariableItem* GetVariablebyId(long Id, bool bConsiderImplicitChoiceElem);  // used by Choice Item
    
    ////////////////////////////////////////////////
    bool Dump(char* filename,bool bOverWrite);
    void Dump(FILE* fp, OTSTR strHeader);
    
    //////////////////////////////////////////////
    virtual CVariableItem*  NewVariableItem();
    
private:
    friend class RunTimeStk;
    friend class CVariableItem;

    static NullSyntaxObject NullObject;
};

/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
// RunTimeProc
enum
{
    CALLBACK_ASSIGNVALUE     =0,    // ݱ()ֵ  A=value 
                                    // PARAM: IdentName
                                    //        Value(CVarient)
        
    CALLBACK_GET_IDENTVALUE  =1,    // ȡݱ()ֵ 
                                    // Ident sample:  A
                                    //                A.B
                                    //                A.B[...].C[..].D ...
                                    //                A[Idx]
                                    //                Func()
                                    //                ConstA::B     -> صProgramȡ
                                    //                ConstA        -> صProgramȡ
                                    // PARAM: IdentName
                                    //        Return: Value(CVarient)
    CALLBACK_SETVALUE        =2,    // ݱ()ֵ  A=szData 
                                    // PARAM: IdentName
                                    //        Value( (char*)szData)
        
};

/////////////////////////////////////////////////////
// SetExStyle(nExStyle)  // FOR CHOICE WHO NAME=NULL
enum 
{ 
    EN_IMPLICIT_CHOICE_ELEM = 1 , // ûĸԺ, ʹûйصGetItemȲʱ, Զ(!!)ʹʽCHOICEĳԱģʽ
};

#ifdef _DLL_PROJECT
class CLASS_EXPORT RunEnv // base
#else
class RunEnv // base
#endif
{
public:
    RunEnv() { m_nExStyle=0; }
    virtual ~RunEnv() {}

    void    setExStyle(WORD nExStyle)  { m_nExStyle=nExStyle; }
    
    virtual cCallback::tFunction GetRunTimeProc() { return NULL; }
    virtual CVariableItem* GetItem(char* varString,long* lastIdx=NULL) { return NULL; }

protected:
    WORD m_nExStyle;

    friend class CVariableItem;
    friend class CStruct;
};

////////////////////////////////////////////
static char* cDumpFormat[]=  
{
    0,          // NULL ʾDefault
    0,
    0,
    "0x%02X",
    "0x%04X",
    "0x%08X",
    0,
    0,
    0,
    0,
    0,
    0
};

static char* cDumpListFormat[]=  
{
    0,          // NULL ʾDefault
    0,
    0,
    "%02X",
    "%04X",
    "%08X",
    0,
    0,
    0,
    0,
    0,
    0
};


#endif

